package org.seasar.cayenne.dao.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectstyle.cayenne.DataObject;
import org.objectstyle.cayenne.dba.TypesMapping;
import org.objectstyle.cayenne.map.DbAttribute;
import org.objectstyle.cayenne.map.DbEntity;
import org.objectstyle.cayenne.map.ObjAttribute;
import org.objectstyle.cayenne.map.ObjEntity;
import org.seasar.cayenne.S2DataContextFactory;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.PropertyDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;

/**
 * @author Takahiro MATSUNAGA
 * @see AbstractAutoCommand
 */
public class InsertCommand extends AbstractAutoCommand {

    /**
     * @param S2DataContextFactory
     * @param beanClass
     */
    public InsertCommand(S2DataContextFactory s2DataContextFactory, Class beanClass) {
        super(s2DataContextFactory, beanClass);
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractAutoCommand#setupSql(org.objectstyle.cayenne.map.ObjEntity)
     */
    protected void setupSql(ObjEntity objEntity) {
        StringBuffer sb = new StringBuffer("insert into ");
        sb.append(objEntity.getDbEntityName()).append(" (");

        StringBuffer sb1 = new StringBuffer();
        StringBuffer sb2 = new StringBuffer();

        DbEntity dbEntity = objEntity.getDbEntity();
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

            sb1.append("#result('");
            sb1.append(objAttr.getName()).append("' '");
            sb1.append(objAttr.getType()).append("'), ");

            sb2.append("#bind($");
            sb2.append(objAttr.getName()).append(" '");
            sb2.append(TypesMapping.getSqlNameByType(dbAttr.getType())).append("'), ");
        }
        sb1.setLength(sb1.length() - 2);
        sb2.setLength(sb2.length() - 2);

        sb.append(sb1);
        sb.append(") values (");
        sb.append(sb2);
        sb.append(")");

        setSql(sb.toString());
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractAutoCommand#setupParams(org.objectstyle.cayenne.map.ObjEntity, org.objectstyle.cayenne.DataObject)
     */
    protected void setupParams(ObjEntity objEntity, DataObject dataObject) {
        Map params = new HashMap();

        BeanDesc beanDesc = BeanDescFactory.getBeanDesc(dataObject.getClass());

        DbEntity dbEntity = objEntity.getDbEntity();
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

            PropertyDesc propDesc = beanDesc.getPropertyDesc(objAttr.getName());
            params.put(objAttr.getName(), propDesc.getValue(dataObject));
        }

        setParams(params);
    }

}
