package test.org.seasar.cayenne.dao.impl;

import java.util.List;

import org.seasar.extension.dataset.DataSet;
import org.seasar.extension.unit.S2TestCase;

import examples.cayenne.dao.EmployeeAutoDao;
import examples.cayenne.dto.EmployeeDto;
import examples.cayenne.entity.Department;
import examples.cayenne.entity.Employee;

/**
 * @author Takahiro MATSUNAGA
 * @see S2TestCase
 */
public class EmployeeAutoDaoTest extends S2TestCase {

    private EmployeeAutoDao employeeDao_;

    /**
     * @param name
     */
    public EmployeeAutoDaoTest(String name) {
        super(name);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(EmployeeAutoDaoTest.class);
    }

    /* (Javadoc Ȃ)
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();

        include("examples/cayenne/dao/EmployeeAutoDao.dicon");
    }

    /* (Javadoc Ȃ)
     * @see junit.framework.TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testGetAllEmployeeTx() {
        readXlsWriteDb("Employee.xls");

        List employees = employeeDao_.getAllEmployee();

        DataSet expected = readXls("getAllEmployee.xls");
        assertEquals("2", expected, employees);
    }

    public void testGetAllEmployeeOrderByEnameTx() {
        readXlsWriteDb("Employee.xls");

        List employees = employeeDao_.getAllEmployeeOrderByEname();

        DataSet expected = readXls("getAllEmployeeOrderByEname.xls");
        assertEquals("1", expected, employees);
    }

    public void testGetEmployeeByIdTx() {
        readXlsWriteDb("Employee.xls");

        Employee employee = employeeDao_.getEmployeeByEmpno(7369);
        Department department = employee.getToDepartment();

        DataSet expected = readXls("getEmployeeById.xls");
        assertEquals("1", expected, employee);
    }

    public void testGetEmployeeByEnameJobTx() {
        readXlsWriteDb("Employee.xls");

        List employee = employeeDao_.getEmployeeByEnameJob("SMITH", "CLERK");

        DataSet expected = readXls("getEmployeeByEnameJob.xls");
        assertEquals("1", expected, employee);
    }

    public void testGetEmployeeByEnameJob2Tx() {
        readXlsWriteDb("Employee.xls");

        EmployeeDto dto = new EmployeeDto();
        dto.setEname("SMITH");
        dto.setJob("CLERK");
        List employee = employeeDao_.getEmployeeByEnameJob2(dto);

        DataSet expected = readXls("getEmployeeByEnameJob.xls");
        assertEquals("1", expected, employee);
    }

    public void testGetEmployeesBySalTx() {
        readXlsWriteDb("Employee.xls");

        List employee = employeeDao_.getEmployeesBySal(1500, 2500);

        DataSet expected = readXls("getEmployeeBySal.xls");
        assertEquals("1", expected, employee);
    }

    public void testInsertTx() {
        readXlsWriteDb("Employee.xls");

        Employee employee = new Employee();
        employee.setEmpno(new Integer(4));
        employee.setEname("XitL");
        employee.setJob("l");

        employeeDao_.insert(employee);

        List employees = employeeDao_.getAllEmployee();

        DataSet expected = readXls("insertEmployee.xls");
        assertEquals("1", expected, employees);
    }

    public void testUpdateTx() {
        readXlsWriteDb("Employee.xls");

        Employee employee = new Employee();
        employee.setEmpno(new Integer(7369));
        employee.setEname("[~");
        employee.setJob("dinot ͔nj");
//        employee.setMgr(null);
//        employee.setHiredate(null);
//        employee.setSal(null);
//        employee.setComm(null);
//        employee.setDeptno(null);

        employeeDao_.update(employee);

        List employees = employeeDao_.getAllEmployee();

        DataSet expected = readXls("updateEmployee.xls");
        assertEquals("1", expected, employees);
    }

    public void testDeleteTx() {
        readXlsWriteDb("Employee.xls");

        Employee employee = new Employee();
        employee.setEmpno(new Integer(7369));

        employeeDao_.delete(employee);

        List employees = employeeDao_.getAllEmployee();

        DataSet expected = readXls("deleteEmployee.xls");
        assertEquals("1", expected, employees);
    }

}
